import { Data } from 'vue';
import { Http, Metadata, IDictionary, Loader } from '@zeedhi/core';
import { Col, Form, ModalService, Card, LoadingService, SelectTree, Select, Button, Header } from '@zeedhi/common';
import html2canvas from "html2canvas";
import { Utilities } from '@/Services/Utilities';
import { CustomAlertService } from '@/Services/CustomAlertService';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import {ChartViewFactory} from "@/Services/ChartView/ChartViewFactory";
import { ExpressionInterpreter } from '@/Services/ExpressionInterpreter';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';
import { IndicatorsController } from './IndicatorsController';

export class ChartViewController {
    public filterModal: any = {};
    public editViewModal: any = null;
    private chartComponent: any = null;
    public viewId: any = null;
    public drillDownFilters: any = {};
    private widget: any = null;
    private viewData: any = null;
    private chart: any = null;
    public indicatorData: any = null;
    public loadingTemplate = {
            "name": "result",
            "component": "ZdImage",
            'height': "100%",
            "src": "../img/loading3.gif"
        };
    public drillDownViewStack: any = [];
    public drillDownFiltersList: any = {};
    public hasDrillDown: boolean = false;
    public drillDownType: string = "";
    public templateData: any = {};
    public drillDownParameterStack: any = [];

    private headerButtons = ["editViewModalButton", "filterModalButton", "exportBtn"];

    public get drilldownViewVisible() {
        return this.hasDrillDown && this.drillDownType === "widget";
    }

    public get drilldownDashboardVisible() {
        return this.hasDrillDown && this.drillDownType === "view";
    }

    public resetDrillStack() {
        this.drillDownParameterStack = [];
    }

    public async apexChartOnCreated(event :any) {
        this.disableHeaderButtons();
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        await this.apexChartOnCreatedAfterLogin(event);
    }

    public async apexChartOnCreatedAfterLogin({ component, editedViewId }:any) {
        const isDrillDown = !!component.parent.parent.fullscreen;
        const urlParams = new URLSearchParams(window.location.search);
        let viewId = urlParams.get('name') ? urlParams.get('name') : '';
        let filters = {};
        let structureId = null;
        let structureName = null;

        if (localStorage.getItem('drillDownView')) {
            viewId = localStorage.getItem('drillDownView');
            localStorage.setItem('drillDownView', "");
        }
        if (localStorage.getItem('drillDownFilter')) {
            filters = JSON.parse(localStorage.getItem('drillDownFilter') || "{}");
            localStorage.setItem('drillDownFilter', "");
        }
        if (localStorage.getItem('drillDownStructure')) {
            const structure = JSON.parse(localStorage.getItem('drillDownStructure') || "{}");
            structureId = structure?.id || null;
            structureName = structure?.name || null;
            localStorage.setItem('drillDownStructure', "");
        }

        if (!viewId && editedViewId) {
            viewId = editedViewId;
        }

        if (typeof viewId === 'string') {
            this.buildSkeletonChart(component);
            this.chartComponent = component;
            this.viewId = viewId;
            await this.buildWidgetByViewId(component, viewId, filters, structureId, structureName);
            if (!editedViewId) {
                this.drillDownParameterStack.push({
                    drillDownView: viewId,
                    drillDownFilters: filters,
                    drillDownStructure: {
                        id: structureId, name: structureName
                    },
                    componentId: component.componentId,
                    widget: this.widget
                });
            }
        }
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonIndicator().children[0]
        ];
    }

    public async cardTitleOnMounted({ component }:any) {
        const urlParams = new URLSearchParams(window.location.search);
        const viewId = urlParams.get('name') ? urlParams.get('name') : '';
        if (viewId) {
            let element: Element | null = document.getElementById(viewId);
            while (element) {
                if (element.className === 'tek-card-title-text') {
                    element.setAttribute('style', component.cssStyle);
                    break;
                }
                element = element.children && element.children[0] ? element.children[0] : null;
            }
        }
    }

    private getViewId() {
        const urlParams = new URLSearchParams(window.location.search);
        if (localStorage.getItem('drillDownView')) {
            return localStorage.getItem('drillDownView');
        }
        if (urlParams.get('name')) {
            return urlParams.get('name');
        }
        return this.viewId;
    }

    public get getExportIndicatorImageFunction() {
        return this.exportIndicatorImage.bind(this, this.getViewId());
    }

    private getImageName() {
        try {
            let name = this.widget[0].widget.name;
            if (this.drillDownParameterStack.length > 0) {
                name = this.drillDownParameterStack[this.drillDownParameterStack.length - 1].widget[0].widget.name || this.widget[0].widget.name;
            }
            return name;
        } catch (e) {
            return "ZmartBI Indicator";
        }
    }

    public async exportIndicatorImage(viewId: any) {
        const canvas = await html2canvas(document.getElementById(viewId || this.viewId)!);
        const image = canvas.toDataURL('image/png');
        Utilities.saveCanvasToPng(image, this.getImageName());
    }

    public replaceAnchorElement({element}: any) {
        const anchorElement = element.querySelector("a.tek-card-title-no-router");
        if (anchorElement) {
            const newElement = document.createElement('p');
            for (let i = 0; i < anchorElement.attributes.length; i += 1) {
                const attr = anchorElement.attributes[i];
                newElement.setAttribute(attr.name, attr.value);
            }
            newElement.innerHTML = anchorElement.innerHTML;
            newElement.classList.add("zd-ma-0");
            anchorElement.replaceWith(newElement);
        }
    }

    public updateHeaderText(clearText: Boolean, textList ?: any, chartViewComponent ?: any) {
        [
            chartViewComponent.parent.parent.children[0].leftSlot[0],
            chartViewComponent.parent.parent.children[0].leftSlot[2],
            chartViewComponent.parent.parent.children[0].leftSlot[4]
        ].forEach((component: any, i: any) => {
            component.text = clearText ? "" : textList[i];
        });
        [
            chartViewComponent.parent.parent.children[0].leftSlot[1],
            chartViewComponent.parent.parent.children[0].leftSlot[3]
        ].forEach((component: any, i: any) => {
            component.isVisible = !clearText && (!!textList[i + 1] && typeof textList[i + 1] === "string");
        });
    }

    private getPeriodString(period: any) {
        if (Array.isArray(period) && period[0]) {
            period = period[0];
        }
        return (period.start + " - " + period.end);
    }

    private disableHeaderButtons() {
        this.headerButtons.forEach((buttonName: any) => {
            const buttons = Metadata.getInstances(buttonName);
            if (buttons && buttons.length > 0) {
                buttons.forEach((button: any) => {
                    button.disabled = true;
                });
            }
        });
    }

    private enableHeaderButtons() {
        this.headerButtons.forEach((buttonName: any) => {
            const buttons = Metadata.getInstances(buttonName);
            if (buttons && buttons.length > 0) {
                buttons.forEach((button: any) => {
                    button.disabled = false;
                });
            }
        });
    }

    private getStructureHeaderText(structures: any) {
        if (Array.isArray(structures)) {
            return structures.join(", ");
        }
        return structures;
    }

    public async buildWidgetByViewId(component: any, viewId: string, filters: any, structureId ?: any, structureName ?: any, preloadedWidget: any = null) {
        if (!filters) {
            filters = {};
        }
        if (!structureId) {
            structureId = "";
        }
        if (!structureName) {
            structureName = "";
        }
        let drillDownFilters = filters;
        if (filters && filters.facts && filters.dimension) {
            drillDownFilters = filters.drillDownFilters;
            if (drillDownFilters) {
                drillDownFilters.period = filters.period;
            }
        }
        let widget = preloadedWidget;
        if (!preloadedWidget) {
            const params = {
                "requestType": "Row",
                "row": {
                    "drillDownFilters": drillDownFilters,
                    "serieColors": 0,
                    "structureId": structureId,
                    "structureName": structureName,
                    "viewId": viewId,
                    "period": filters.period ? filters.period : null,
                    "dimension": filters.dimension ? filters.dimension : null,
                    "facts": filters.facts ? filters.facts : null,
                    "isChartView": true,
                    "dashboardParameters": filters.dashboardParameters ? filters.dashboardParameters : null,
                }
            };
            const response = await Http.post("/buildWidgetByViewIdNext", params);
            widget = response.data?.dataset?.widget;
            console.log(widget);
            if (response.data?.dataset?.error && response.data?.dataset?.error.length > 0) {
                Utilities.widgetErrorMessageAlerts(response.data.dataset.error);
            }
            if (response.data?.error) {
                Utilities.widgetErrorMessageAlerts([response.data.error]);
            }
        }
        this.drillDownFilters = filters;
        component.children = [];
        if (widget) {
            this.enableHeaderButtons();
            this.widget = widget;
            if (widget[0].name === 'notFound') {
                setTimeout(function () {
                    component.children = widget;
                }, 0);
                this.chart = ZmartChartsApex.build(widget, this, component);
            } else {
                if (viewId !== this.viewId) {
                    this.viewId = viewId;
                }
                this.chart = ZmartChartsApex.build(widget, this, component);

                const structureText = structureName ? this.getStructureHeaderText(structureName) : this.getStructureHeaderText(widget[0].widget.parameters.structureName);
                if (!filters.period && widget[0] && widget[0].widget && widget[0].widget.parameters && widget[0].widget.parameters.period && widget[0].widget.parameters.period && widget[0].widget.parameters.period.start && widget[0].widget.parameters.period.end) {
                    filters.period = widget[0].widget.parameters.period;
                }
                const periodText = filters.period ? this.getPeriodString(filters.period) : "Todo o período";
                const drillText = this.drillDownFilters ? (Object.values(this.drillDownFilters).filter((value: any) => typeof value === 'string')) : "";
                this.updateHeaderText(false, [structureText, periodText, (typeof drillText === 'object' ? drillText.join(" - ") : drillText)], component);
                if (filters) {
                    if (filters.drillDownFilters && Object.values(filters.drillDownFilters).length > 0) {
                        filters = {...filters, ...filters.drillDownFilters};
                    }
                    this.chart.chartInstance.period = filters.period;
                    if (Array.isArray(filters.period) && filters.period[0]) {
                        this.chart.chartInstance.period = filters.period;
                    }
                    const drillFiltersFieldValue: any = [];
                    Object.entries(filters).forEach(function ([key, value]: any) {
                        if (!["period", "drillDownFilters", "facts", "dimension"].includes(key)) {
                            drillFiltersFieldValue.push({ name: key, value });
                        }
                    });
                    this.chart.chartInstance.drillDownFilters = drillFiltersFieldValue;
                }
                const widgetChats = this.chart.getCharts();
                component.children = widgetChats;
            }
        }
    }

    protected evalCondition(condition: String, variables: any) {
        condition = condition.replaceAll('row.', '');
        const operators: any = {
          '==': (a: any, b: any) => a === b,
          '!=': (a: any, b: any) => a !== b,
          '<': (a: any, b: any) => a < b,
          '>': (a: any, b: any) => a > b,
          '<=': (a: any, b: any) => a <= b,
          '>=': (a: any, b: any) => a >= b,
        };

        const conditions = condition.split(/&&|\|\|/);
        const conditionComparators: any = condition.match(/(\|\||&&)/g);

        const results = conditions.map((singleCondition: any) => {
            if (singleCondition === "true") {
                return true;
            }
            if (singleCondition === "false") {
                return false;
            }
            const parts = singleCondition.match(/\S+/g);

            if (parts.length !== 3) {
                console.error("Condição inválida: ", singleCondition);
                return false;
            }

            const [leftOperand, operator, rightOperand] = parts;
            let leftValue = variables[leftOperand];
            if (leftValue === undefined) {
                leftValue = parseInt(leftOperand, 10);
            }
            let rightValue = variables[rightOperand];
            if (rightValue === undefined) {
                rightValue = parseInt(rightOperand, 10);
            }
            if (leftValue === undefined || rightValue === undefined) {
                console.error("Condition error: ", singleCondition);
                return false;
            }
            return operators[operator](leftValue, rightValue);
        });
        if (conditionComparators) {
            while (conditionComparators.length > 0) {
                const comparator = conditionComparators.shift();
                const a = results.shift();
                const b = results.shift();
                if (comparator === "||") {
                    results.unshift(a || b);
                }
                if (comparator === "&&") {
                    results.unshift(a && b);
                }
            }
        }
        return results[0];
    }

    protected cssStringToJSON(cssString: any) {
        const cssRegex = /([^:;]+):([^;]+);/g;
        const cssObject : any = {};
        let match = cssRegex.exec(cssString);
        while (match !== null) {
          const propertyName = match[1].trim();
          const propertyValue = match[2].trim();

          cssObject[propertyName] = propertyValue;
          match = cssRegex.exec(cssString);
        }
        return cssObject;
      }

    public applyGridStylesCrosstab(crosstab: any, cell: any) {
        const row: any = {};
        const gridStyles = this.chart.chartInstance.zmartCharts.chartMetadata[0].children[1].widget.gridStyle;
        const column = this.chart.chartInstance.zmartCharts.chartMetadata[0].children[1].widget.yKeys;
        cell.row.forEach(function (r: any) {
            row[r] = cell.value;
        });
        return this.applyStyles(gridStyles, column, row);
    }

    public applyGridStyles({row, column}: IDictionary<any>) {
        let gridStylesField = {};
        if (this.chart) {
            const gridStyles = this.chart.chartInstance.zmartCharts.chartMetadata[0].widget.gridStyle;
            gridStylesField = this.applyStyles(gridStyles, column, row);
        }
        return gridStylesField;
    }

    protected columnCompare(column: any, styles: any) {
        if (Array.isArray(column)) {
            return column.some((element: any) => styles.includes(element));
        }
        return styles.includes(column);
    }

    protected applyStyles(gridStyles: any, column: any, row: any) {
        if (gridStyles) {
            let cssStyle: any = {};
            gridStyles.forEach((style: any) => {
                if (this.columnCompare(column, style.styleFields) && this.evalCondition(style.condition, row)) {
                    const code = style.css.cssCode.match(/{([^}]*)}/);
                    if (code) {
                        cssStyle = {...cssStyle, ...this.cssStringToJSON(code[1])};
                    }
                }
            });
            return cssStyle;
        }
        return {};
    }

    public backButton() {
        this.chart.goBackDrill();
    }

    public get getOpenEditViewModalFunction() {
        return this.openEditViewModal.bind(this, this.getViewId());
    }

    public async openEditViewModal(viewId: any) {
        if (viewId !== this.viewId) {
            await this.refreshParameters(viewId);
        }
        this.createEditViewModal(viewId);
    }

    private async getIndicatorData(viewData: any) {
        const response = await Http.get("/indicator");
        this.indicatorData = response.data.data.find((v: any) => v._id === viewData.indicator);
    }

    public async fillView(viewTypeId: any) {
        const chart = ChartViewFactory.getInstanceChartType(viewTypeId);
        const viewData = chart.getEditData(this.viewData);
        const form = Metadata.getInstance<Form>("viewForm");
        const dateFilterField = Metadata.getInstance<Select>("dateFilterColumn");
        const manualValueSetFields = ["period", "hasDrillDown", "drillDownType"];
        await this.getIndicatorData(viewData);
        form.clearForm();
        Object.keys(viewData).filter((key) => !manualValueSetFields.includes(key)).forEach((key) => {
            setTimeout(() => {
                form.value[key] = viewData[key];
            }, 0);
        });
        this.hasDrillDown = viewData.hasDrillDown;
        this.drillDownType = viewData.drillDownType;
        this.generatePeriods(viewData.period);
        await dateFilterField.datasource.updateData(this.indicatorData.dataSourceColumns.filter((v: any) => v.isDate) || []);
        dateFilterField.setValue(viewData.dateFilterColumn);
    }

    public async generatePeriods(periods: any = [undefined]) {
        const form = Metadata.getInstance<Form>("viewForm_period");
        form.clearForm();
        if (!Array.isArray(periods)) {
            periods = [periods];
        }
        await this.generatePeriodHelperOptions();
        form.childrenInstances.forEach((el: any) => form.removeChild(el.name));
        form.children = [];
        form.childrenInstances = [];
        form.children = periods.map((period: any, i: number) => this.createPeriodElement(period, periods.length - 1 === i)).flat();
    }

    public adjustPeriodDropdown({ component }: any) {
        const value = component.helperValue;
        if (typeof value === 'string') {
            if (component.helperOptions?.includes(value)) {
                component.updateHelperHint(value);
            }
        }
        // Diminui o tamanho do menu de data
        const periodDropdown = component.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
    }

    private periodElementId: number = -1;
    public createPeriodElement(period: any = null, isLast: boolean = false) {
        this.periodElementId += 1;
        const periodFormat = Utilities.getDateFormatField(period, ' - ');
        let helperValue = "";
        if (typeof period === 'string') {
            helperValue = period;
        }
        return [{
            "name": "period_" + this.periodElementId,
            "component": "ZdDateRange",
            "label": "Período",
            "grid": { "cols": 7 },
            "value": periodFormat,
            "dateFormat": "DD/MM/YYYY",
            "inputFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "splitter": " - ",
            "orderedDates": true,
            "helperOptions": this.periodData,
            "helperValue": helperValue,
            "events": { "onMounted": this.adjustPeriodDropdown.bind(this) },
            "validations": { "dateRangeValidation": {} }
        },
        {
            "name": "periodButtons_" + this.periodElementId,
            "component": "ZdContainer",
            "cssClass": "zd-pa-0",
            "children": [
                {
                    "name": "addButton_" + this.periodElementId,
                    "component": "ZdButton",
                    "label": "Adicionar outro periodo",
                    "iconName": "mdi-plus",
                    "isVisible": isLast,
                    "events": { "click": this.addPeriod.bind(this) }
                },
                {
                    "name": "deleteButton_" + this.periodElementId,
                    "component": "ZdButton",
                    "label": "Remover periodo",
                    "iconName": "mdi-delete",
                    "isVisible": !isLast,
                    "events": { "click": this.removePeriod.bind(this) }
                }
            ]
        }];
    }

    public updatePeriodStatus() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        form.children.forEach((el: any, i: number) => {
            if (el.name.startsWith("periodButtons")) {
                if (i === form.children.length - 1) {
                    el.children[0].isVisible = true;
                    el.children[1].isVisible = false;
                } else {
                    el.children[0].isVisible = false;
                    el.children[1].isVisible = true;
                }
            }
        });
    }

    public addPeriod() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        const period = this.createPeriodElement();
        form.addChild(period[0]);
        form.addChild(period[1]);
        this.updatePeriodStatus();
    }

    public removePeriod({ component }: any) {
        const form = Metadata.getInstance<Form>("viewForm_period");
        const id = component.name.replace("deleteButton_", "");
        form.removeChild("period_" + id);
        form.removeChild("periodButtons_" + id);
        if (form.value["period_" + id]) {
            delete form.value["period_" + id];
        }
    }

    private periodData: any = [];
    public async generatePeriodHelperOptions() {
        const periodEls = Metadata.getInstance<Form>("viewForm_period").children;

        periodEls.forEach((period: any) => {
            Utilities.unregisterPeriod(period);
        });

        this.periodData = await Utilities.getHelperOptions();
    }

    public async getViewFormFields(viewTypeId: any) {
        const chart = ChartViewFactory.getInstanceChartType(viewTypeId);
        const suggestedColors = await Utilities.getArrayColors();
        const viewData = chart.getEditData(this.viewData);
        if (viewData.chartLegendClosed) {
            viewData.legendClosed = viewData.chartLegendClosed;
        }
        if (viewData.zmartSerieRanges) {
            viewData.serieRanges = viewData.zmartSerieRanges;
        }
        const ZOrderCheck = (a: any, b: any) => {
            if (a.ZOrder && b.ZOrder) {
                return a.ZOrder - b.ZOrder;
            }
            return -1;
        };
        const formExtra = Metadata.getInstance<Form>("viewForm_extraFields");
        formExtra.clearForm();
        formExtra.childrenInstances.forEach((el: any) => formExtra.removeChild(el.name));
        formExtra.children = [];
        formExtra.childrenInstances = [];

        const viewTypes = await Http.get("/viewTypeNext");
        let properties = viewTypes.data.data.filter((viewType: any) => viewType._id === viewTypeId);
        properties = properties.length > 0 ? properties[0].properties : [];

        const res = await Metadata.get("indicators/view/viewDetailMega", true);
        const fieldsShown = res.children.filter((field: any) => properties.includes(field._id)).sort(ZOrderCheck);
        const groups: any = {};
        const noGroup: any = [];
        const groupsNames: any = res.ZFieldGroups;

        fieldsShown.forEach((field: any) => {
            if (field.datasource && field.datasource.filter && field.datasource.filter._id) {
                field.datasource.filter._id = viewData.indicator;
            }

            if (field.component === "ZdColorPicker") {
                field.swatches = suggestedColors;
            }

            if (field.ZFieldGroup) {
                if (!groups[field.ZFieldGroup]) {
                    groups[field.ZFieldGroup] = [];
                }

                groups[field.ZFieldGroup].push(field);
            } else {
                noGroup.push(field);
            }
        });

        const fieldsWithGroup: any = Object.entries(groups).map(([key, value]: [any, any]) => {
            const title = {
                "name": key + "_title",
                "component": "ZdText",
                "text": key in groupsNames ? groupsNames[key] : key,
                "tag": "h3",
                "grid": { "cols": 12 },
            };
            const divider = {
                "name": key + "_divider",
                "component": "ZdDivider",
                "grid": { "cols": 12 },
            };

            groups[key] = value.sort(ZOrderCheck);
            return [title, divider, ...groups[key]];
        });
        const fields = [...noGroup, ...([].concat(...fieldsWithGroup))];
        if (fields.length > 0) {
            fields.forEach((field: any) => {
                if (field.name in viewData) {
                    field.value = viewData[field.name];
                }
            });
        }
        formExtra.children = fields;
    }

    public updateViewType({component}: any) {
        if (component.datasource.currentRow) {
            this.templateData.template = component.datasource.currentRow.template;
            this.templateData.name = component.datasource.currentRow.name;
            this.templateData._id = component.datasource.currentRow._id;
            const type = component.value;
            const chart = ChartViewFactory.getInstanceChartType(this.viewData.viewTypeId);
            const viewData = chart.getEditData(this.viewData);
            if (type !== viewData.viewTypeId) {
                this.getViewFormFields(type);
            }
        }
    }

    public getPeriodsValue() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        let value = form.childrenInstances.filter((el: any) => el.name.startsWith("period_") && el).map((el: any) => el.helperValue || form.value[el.name]);
        value = value.filter((el: any) => !Array.isArray(el) || el.length === 2).map((el: any) => {
            if (Array.isArray(el)) {
                return { start: el[0], end: el[1] };
            }
            return el;
        });
        if (value.length === 1) {
            return value[0];
        }
        return value;
    }

    public prepareWidgetToSave(formExtra: any, values: any, cleanValues: any) {
        if (!values.hasDrillDown || values.drillDownType === 'structure') {
            cleanValues.drillDownViewField = null;
            cleanValues.drillDownView = null;
            cleanValues.drillDownDashboardField = null;
        } else if (values.hasDrillDown) {
            if (values.drillDownType === 'widget') {
                cleanValues.drillDownDashboardField = null;
            } else if (values.drillDownType === 'view') {
                cleanValues.drillDownViewField = null;
                cleanValues.drillDownView = null;
            }
        }
        cleanValues.template = this.templateData.template;
        cleanValues.chartType = this.templateData.name;
        cleanValues.viewType = this.templateData.name;
        cleanValues.serieType = this.templateData.name;
        cleanValues.columns = this.indicatorData.dataSourceColumns;
        const chart = ChartViewFactory.getInstanceChartType(cleanValues.viewTypeId);
        chart.setEditData(this.viewData);
        cleanValues = chart.getSaveViewParams(formExtra, values, cleanValues);
        return cleanValues;
    }

    public getConditionExpression(dataSourceColumns: any, conditionString: any, previousOrPost: any) {
        if (!conditionString) return null;
        const columnsNames = dataSourceColumns.map(function (column: { name: any; }) {
            return column.name;
        });
        return ExpressionInterpreter.readComparisonExpression(conditionString, previousOrPost, columnsNames, columnsNames);
    }

    public async saveView() {
        const form = Metadata.getInstance<Form>("viewForm");
        const formExtra = Metadata.getInstance<Form>("viewForm_extraFields");
        const formPeriod = Metadata.getInstance<Form>("viewForm_period");
        const period = this.getPeriodsValue();

        if (!form.validate() || !formExtra.validate() || !formPeriod.validate()) {
            return;
        }

        const previousConditions = this.getConditionExpression(this.indicatorData.dataSourceColumns, form.value.previousConditionsString || null, "previous");
        const postConditions = this.getConditionExpression(this.indicatorData.dataSourceColumns, form.value.postConditionsString || null, "post");
        const postGroupConditions = this.getConditionExpression(this.indicatorData.dataSourceColumns, form.value.postGroupConditionsString || null, "postGroup");

        const values: any = {
            ...form.value,
            ...formExtra.value,
            period
        };
        let cleanValues = Object.fromEntries(Object.entries(values).filter(([key, value]: [string, any]) => value != null));
        cleanValues = {
            ...cleanValues,
            previousConditions,
            postConditions,
            postGroupConditions,
        };
        this.prepareWidgetToSave(formExtra, values, cleanValues);
        try {
            LoadingService.show();
            const res = await Http.post("/widgetsWithIndicatorNames/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [cleanValues],
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            this.editViewModal.hide();
            this.apexChartOnCreated({component: this.chartComponent, editedViewId: cleanValues._id});
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public async createEditViewModal(viewId: any) {
        const modal = ModalService.getModal('viewConfigCard');
        if (modal) {
            modal?.destroy();
        }
        if (this.editViewModal) {
            this.editViewModal?.destroy();
        }
        LoadingService.show();
        const editModal = await ModalService.createFromJson('indicators/view/viewDetail_chartView', true);
        if (editModal) {
            editModal!.children![1].children![0].cssClass = "zd-pa-0";
            editModal!.children![1].children![0].cssStyle = "height: calc(100vh - 240px);";
            this.editViewModal = ModalService.create(editModal.children[1]);
            this.editViewModal.show();
            setTimeout(() => {
                this.loadFilterFields(viewId);
            }, 0);
        }
    }

    private async loadFilterFields(viewId: any) {
        const resWidget = await Http.post("/widget", {
            requestType: "FilterData",
            disableLoader: false,
            filter: [
                {
                    name: "_id",
                    operator: "=",
                    value: viewId || this.widget[0].widget.id
                }
            ],
        });

        this.viewData = this.widget[0].widget;
        if (resWidget && resWidget.data && resWidget.data.dataset && resWidget.data.dataset.widget && resWidget.data.dataset.widget[0]) {
            this.viewData = resWidget.data.dataset.widget[0];
        }
        await this.fillView(this.viewData.viewTypeId);
        await this.getViewFormFields(this.viewData.viewTypeId);
        LoadingService.hide();
    }

    public closeEditViewModal() {
        this.editViewModal.hide();
        this.editViewModal?.destroy();
    }

    public get getOpenFilterModalFunction() {
        return this.openFilterModal.bind(this, this.getViewId());
    }

    private async refreshParameters(viewId: any) {
        LoadingService.show();
        let drillStack = {
            drillDownFilters: {},
            drillDownStructure: {id: null, name: null},
            componentId: this.chartComponent.componentId,
            widget: null
        };
        if (this.drillDownParameterStack.length > 0) {
            drillStack = this.drillDownParameterStack[this.drillDownParameterStack.length - 1];
        }
        if (drillStack.componentId && this.chartComponent.componentId !== drillStack.componentId) {
            const cards = Metadata.getInstances("card-view-chart");
            this.chartComponent = cards.find((card: any) => card.componentId === drillStack.componentId);
        }
        await this.buildWidgetByViewId(this.chartComponent, viewId, drillStack.drillDownFilters || {}, drillStack.drillDownStructure.id || null, drillStack.drillDownStructure.name || null, drillStack.widget || null);
        LoadingService.hide();
    }

    public async openFilterModal(viewId: any) {
        if (viewId !== this.viewId) {
            await this.refreshParameters(viewId);
        }
        this.createFilterModal();
    }

    public async createFilterModal() {
        LoadingService.show();
        if (this.filterModal && !!this.filterModal.destroy) {
            this.filterModal?.destroy();
        }
        const modal = await this.chart.chartInstance.getFilterModal();
        this.filterModal = ModalService.create(modal);
        this.filterModal.show();
        LoadingService.hide();
    }

    public closeFilterModal() {
        this.filterModal.hide();
        this.filterModal?.destroy();
    }

    public async applyFilter() {
        this.updateHeaderText(true, [], Metadata.getInstance<Card>("card-view-chart"));
        const form = Metadata.getInstance<Form>('filterForm');
        if (!form.validate()) {
            return;
        }
        const filters: any = {};
        let structureId: any = null;
        let structureName = null;
        filters.period = {};
        if (!!form.value.period && !!form.value.period.length) {
            const dateStart = form.value.period[0];
            const dateEnd = form.value.period[1];
            filters.period = {
                "start": dateStart,
                "end": dateEnd
            };
        }

        const periods: any = [];
        Object.keys(form.value).forEach((key: any) => {
            if (key.startsWith('period_') && form.value[key] && form.value[key].length && form.value[key].length === 2) {
                periods.push({
                    start: form.value[key][0],
                    end: form.value[key][1],
                });
            }
        });

        if (periods.length > 0) {
            if (Object.keys(filters.period).length > 0) {
                filters.period = [filters.period, ...periods];
            } else {
                filters.period = periods;
            }
        }

        if (form.value.drillDownFilters) {
            filters.drillDownFilters = {};
            form.value.drillDownFilters.reduce((acc: any, obj: any) => {
                acc[obj.name] = obj.value;
                return acc;
            }, filters.drillDownFilters);
        }

        if (form.value.structure_label !== undefined) {
            structureId = form.value.structure_label;
            const selectComponent = Metadata.getInstance<SelectTree>("structure_label");
            if (selectComponent) {
                const structureRow: any = selectComponent?.datasource?.data.find((data: any) => data._id === structureId);
                structureName = structureRow.name;
            }
        }

        if (form.value.dimensionFilter) {
            filters.dimension = form.value.dimensionFilter;
        }

        if (form.value.factsFilter) {
            filters.facts = form.value.factsFilter;
        }

        Object.entries(form.value).forEach(([key, value]: any) => {
            if (value && value.length && value.length > 0 && key.startsWith("filter_")) {
                filters[key.replace("filter_", "")] = value;
            }
        });

        this.closeFilterModal();
        this.buildSkeletonChart(this.chartComponent);
        await this.buildWidgetByViewId(this.chartComponent, this.viewId, filters, structureId, structureName);
    }

    public static openEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = true;
        colViewEdit.cols = 4;
        colViewEdit.sm = 4;
        colViewEdit.md = 4;
        colViewEdit.lg = 4;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 8;
        colViewChart.sm = 8;
        colViewChart.md = 8;
        colViewChart.lg = 8;
        ChartViewController.createFieldsEditView().then();
    }

    public static closeEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = false;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 12;
        colViewChart.sm = 12;
        colViewChart.md = 12;
        colViewChart.lg = 12;
    }

    public static async createFieldsEditView() {
        const params = {
            "requestType": "FilterData",
            "filter": {}
        };
        const response = await Http.post("/viewProperty", params);
        if (response.data.dataset.viewProperty) {
            const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
            response.data.dataset.viewProperty.forEach(function (viewProperty: any) {
                if (viewProperty) {
                    viewProperty.component = "ZdTextInput";
                }
            });
        }
    }

    public static setFieldsEditView(typeView: any) {
        const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
        colViewEdit.children.forEach(function (field: any) {
            if (typeView.properties.indexOf(field._id) !== -1) {
                field.isVisible = true;
            } else {
                field.isVisible = false;
            }
        });
    }

    private getValueFromRow(row: any) {
        return row.infinity ? Infinity : parseFloat(row.changeValue);
    }

    public periodCompareChangeActionGetVisibility({row}: any) {
        return !!row.change;
    }

    public periodCompareChangeActionGetColor({row}: any) {
        const value = this.getValueFromRow(row);
        if (value < 0) {
            return 'error';
        }
        if (value > 0) {
            return 'success';
        }
        return 'grey';
    }

    public periodCompareChangeActionGetIconName({row}: any) {
        const value = this.getValueFromRow(row);
        if (value < 0) {
            return 'mdi-chevron-down';
        }
        if (value > 0) {
            return 'mdi-chevron-up';
        }
        return '';
    }

    public periodCompareChangeActionGetLabel({row}: any) {
        const value = this.getValueFromRow(row);
        return (value === Infinity ? '∞' : Math.abs(value)) + '%';
    }
}
